#include "stdafx.h"
#include "SQLViewApp.h"
#include "SQLTableDoc.h"
#include "SQLTableView.h"
#include "ColumnFormatDlg.h"
#include "ErrorBox.h"
#include "OpenDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


BEGIN_MESSAGE_MAP(COpenDlg, CDialog)
	//{{AFX_MSG_MAP(COpenDlg)
	ON_LBN_DBLCLK(ID_OPEN_TABLES, OnSelectTable)
	ON_BN_CLICKED(ID_OPEN_SELECTTABLE, OnSelectTable)
	ON_LBN_DBLCLK(ID_OPEN_FIELDS, OnAdd)
	ON_BN_CLICKED(ID_OPEN_SETDATAFIELD, OnSetDataField)
	ON_BN_CLICKED(ID_OPEN_ADD, OnAdd)
	ON_BN_CLICKED(ID_OPEN_REMOVE, OnRemove)
	ON_LBN_DBLCLK(ID_OPEN_COLUMNS, OnColumnFormat)
	ON_BN_CLICKED(ID_OPEN_COLUMNFORMAT, OnColumnFormat)
	ON_EN_KILLFOCUS(ID_OPEN_SORTFORMAT, OnKillFocusSortFormat)
	ON_BN_CLICKED(ID_OPEN_OK, OnOK)
	ON_BN_CLICKED(ID_OPEN_CANCEL, OnCancel)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


COpenDlg::COpenDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COpenDlg::IDD, pParent),
	m_pSQLTableDoc(NULL)
{
	//{{AFX_DATA_INIT(COpenDlg)
	//}}AFX_DATA_INIT
}


COpenDlg::COpenDlg(CWnd* pParent, CSQLTableDoc* pSQLTableDoc)
	: CDialog(COpenDlg::IDD, pParent),
	m_pSQLTableDoc(pSQLTableDoc)
{
}


COpenDlg::~COpenDlg()
{
	m_arrColumns.RemoveAll();
}


BOOL COpenDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CString sQuery = CString(_T("SELECT Name FROM SysObjects WHERE Type = 'U' AND SysStat & 0xf = 3 ORDER BY Name"));
	m_pSQLTableDoc->Open(sQuery);
	while (!m_pSQLTableDoc->GetEOF())
	{
		CString s = LPCTSTR(_bstr_t(m_pSQLTableDoc->GetField(_T("Name"))));
		m_ctrlTables.AddString(s);

		m_pSQLTableDoc->MoveNext();
	}
	m_pSQLTableDoc->Close();

	if (m_ctrlTables.GetCount() > 0)
		m_ctrlTables.SetCurSel(0);
	
	m_ctrlSortFormat.SetWindowText(_T("%"));
	m_ctrlOptimizerHints.SetWindowText(_T("(INDEX = )"));
	m_ctrlShowVertGridLines.SetCheck(1);
	m_ctrlEnableIteratorBar.SetCheck(1);

	m_ctrlTables.SetFocus();
	
	return TRUE;
}


void COpenDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COpenDlg)
	DDX_Control(pDX, ID_OPEN_TABLES, m_ctrlTables);
	DDX_Control(pDX, ID_OPEN_SELECTTABLE, m_ctrlSelectTable);
	DDX_Control(pDX, ID_OPEN_FIELDS, m_ctrlFields);
	DDX_Control(pDX, ID_OPEN_SETDATAFIELD, m_ctrlSetDataField);
	DDX_Control(pDX, ID_OPEN_DATAFIELD, m_ctrlDataField);
	DDX_Control(pDX, ID_OPEN_ADD, m_ctrlAdd);
	DDX_Control(pDX, ID_OPEN_REMOVE, m_ctrlRemove);
	DDX_Control(pDX, ID_OPEN_COLUMNS, m_ctrlColumns);
	DDX_Control(pDX, ID_OPEN_COLUMNFORMAT, m_ctrlColumnFormat);
	DDX_Control(pDX, ID_OPEN_SORT, m_ctrlSort);
	DDX_Control(pDX, ID_OPEN_SORTFORMAT, m_ctrlSortFormat);
	DDX_Control(pDX, ID_OPEN_OPTIMIZERHINTS, m_ctrlOptimizerHints);
	DDX_Control(pDX, ID_OPEN_FILTER, m_ctrlFilter);
	DDX_Control(pDX, ID_OPEN_FIXEDCOLUMNS, m_ctrlFixedColumns);
	DDX_Control(pDX, ID_OPEN_SHOWVERTGRIDLINES, m_ctrlShowVertGridLines);
	DDX_Control(pDX, ID_OPEN_ENABLEITERATORBAR, m_ctrlEnableIteratorBar);	
	DDX_Control(pDX, ID_OPEN_OK, m_ctrlOK);
	DDX_Control(pDX, ID_OPEN_CANCEL, m_ctrlCancel);
	//}}AFX_DATA_MAP
}


void COpenDlg::OnSelectTable()
{
	m_ctrlTables.GetText(m_ctrlTables.GetCurSel(), m_sTableName);

	m_ctrlFields.ResetContent();

	CString sQuery = CString(_T("SELECT Columns.Name FROM SysColumns Columns, SysObjects Objects WHERE Objects.Name = '")) +
		m_sTableName +
		CString(_T("' AND Columns.Id = Objects.Id"));

	m_pSQLTableDoc->Open(sQuery);
	while (!m_pSQLTableDoc->GetEOF())
	{
		CString s = LPCTSTR(_bstr_t(m_pSQLTableDoc->GetField(_T("Name"))));

		// set item data == item index
		m_ctrlFields.AddString(s);
		int iItemIndex = m_ctrlFields.GetCount() - 1;
		m_ctrlFields.SetItemData(iItemIndex, iItemIndex);

		m_pSQLTableDoc->MoveNext();
	}
	m_pSQLTableDoc->Close();

	m_ctrlFields.SetCurSel(0);

	// Enable all dlg items
	CWnd* pWnd = GetWindow(GW_CHILD);
	while (pWnd)
	{
		pWnd->EnableWindow();
		pWnd = pWnd->GetWindow(GW_HWNDNEXT);
	}

	m_ctrlTables.SetFocus();
}


void COpenDlg::OnSetDataField() 
{
	CString sDataFieldName;
	m_ctrlFields.GetText(m_ctrlFields.GetCurSel(), sDataFieldName);
	
	m_ctrlDataField.SetWindowText(sDataFieldName);
}


void COpenDlg::OnAdd()
{
	if (m_ctrlFields.GetCount() == 0)
		return;
	
	int iSel = m_ctrlFields.GetCurSel();
	
	CString sFieldName;
	m_ctrlFields.GetText(iSel, sFieldName);
	sFieldName.TrimRight();

	CColumnData ColumnData;
	ColumnData.sFieldName = sFieldName;
	ColumnData.sFieldFormat = CString(_T("%"));
	ColumnData.sHeading = sFieldName;
	ColumnData.iFormat = NULL;
	ColumnData.iWidth = 128;

	CColumnFormatDlg dlgColumnFormat(NULL, ColumnData);
	if (dlgColumnFormat.DoModal() == IDOK)
	{
		ColumnData.sFieldName = dlgColumnFormat.m_sFieldName;
		ColumnData.sFieldFormat = dlgColumnFormat.m_sFieldFormat;
		ColumnData.sHeading = dlgColumnFormat.m_sHeading;
		ColumnData.iFormat = dlgColumnFormat.m_iFormat;
		ColumnData.iWidth = dlgColumnFormat.m_iWidth;
		
		DWORD dwData = m_ctrlFields.GetItemData(iSel);

		m_arrColumns.Add(ColumnData);
		m_ctrlColumns.AddString(sFieldName);
		m_ctrlColumns.SetItemData(m_ctrlColumns.GetCount() - 1, dwData);
		m_ctrlColumns.SetCurSel(m_ctrlColumns.GetCount() - 1);

		m_ctrlFields.DeleteString(iSel);
		
		int iCount = m_ctrlFields.GetCount();
		if (iCount > 0)
		{
			if (iSel <= (iCount - 1))
				m_ctrlFields.SetCurSel(iSel);
			else
				m_ctrlFields.SetCurSel(iCount - 1);
		}
	}
}


void COpenDlg::OnRemove() 
{
	if (m_ctrlColumns.GetCount() == 0)
		return;
	
	int iSel = m_ctrlColumns.GetCurSel();
	
	CString sFieldName;
	m_ctrlColumns.GetText(iSel, sFieldName);
	DWORD dwData = m_ctrlColumns.GetItemData(iSel);

	m_arrColumns.RemoveAt(iSel);
	m_ctrlColumns.DeleteString(iSel);

	int iCount = m_ctrlColumns.GetCount();
	if (iCount > 0)
	{
		if (iSel <= (iCount - 1))
			m_ctrlColumns.SetCurSel(iSel);
		else
			m_ctrlColumns.SetCurSel(iCount - 1);
	}

	iCount = m_ctrlFields.GetCount();
	if (iCount > 0)
	{
		int i;
		for (i = 0; i < iCount; i++)
		{
			if (dwData < m_ctrlFields.GetItemData(i))
			{
				m_ctrlFields.InsertString(i, sFieldName);
				m_ctrlFields.SetItemData(i, dwData);
				m_ctrlFields.SetCurSel(i);
				break;
			}
		}
		
		if (i == iCount)
		{
			// item was not inserted, so it will be last
			m_ctrlFields.AddString(sFieldName);
			m_ctrlFields.SetItemData(m_ctrlFields.GetCount() - 1, dwData);
			m_ctrlFields.SetCurSel(m_ctrlFields.GetCount() - 1);
		}
	}
	else
	{
		m_ctrlFields.InsertString(0, sFieldName);
		m_ctrlFields.SetItemData(0, dwData);
		m_ctrlFields.SetCurSel(0);
	}
}


void COpenDlg::OnColumnFormat()
{
	if (m_ctrlColumns.GetCount() == 0)
		return;
	
	int iSel = m_ctrlColumns.GetCurSel();
	CColumnData ColumnData = m_arrColumns.GetAt(iSel);

	CColumnFormatDlg dlgColumnFormat(NULL, ColumnData);
	if (dlgColumnFormat.DoModal() == IDOK)
	{
		ColumnData.sFieldName = dlgColumnFormat.m_sFieldName;
		ColumnData.sFieldFormat = dlgColumnFormat.m_sFieldFormat;
		ColumnData.sHeading = dlgColumnFormat.m_sHeading;
		ColumnData.iFormat = dlgColumnFormat.m_iFormat;
		ColumnData.iWidth = dlgColumnFormat.m_iWidth;

		m_arrColumns.SetAt(iSel, ColumnData);
	}
}


void COpenDlg::OnKillFocusSortFormat() 
{
	CString sSortFormat;
	m_ctrlSortFormat.GetWindowText(sSortFormat);

	if (!CheckFormat(sSortFormat))
		m_ctrlSortFormat.SetFocus();
}


void COpenDlg::OnOK() 
{
	m_ctrlDataField.GetWindowText(m_sDataFieldName);
	m_sDataFieldName.TrimRight();
	if (m_sDataFieldName.IsEmpty())
	{
		CErrorBox("", _T("Fill Data Field"));
		m_ctrlSetDataField.SetFocus();
		return;
	}

	if (m_ctrlColumns.GetCount() == 0)
	{
		CErrorBox("", _T("Add some columns"));
		m_ctrlAdd.SetFocus();
		return;
	}

	m_ctrlSort.GetWindowText(m_sSort);
	m_sSort.TrimRight();
	if (m_sSort.IsEmpty())
	{
		CErrorBox("", _T("Fill ORDER BY clause"));
		m_ctrlSort.SetFocus();
		return;
	}

	m_ctrlSortFormat.GetWindowText(m_sSortFormat);
	if (!CheckFormat(m_sSortFormat))
	{
		CErrorBox("", _T("Bad ORDER BY format"));
		m_ctrlSortFormat.SetFocus();
		return;
	}

	if (!m_ctrlOptimizerHints.GetModify())
	{
		CErrorBox("", _T("Bad Optimizer Hints"));
		m_ctrlOptimizerHints.SetFocus();
		return;
	}

	m_ctrlOptimizerHints.GetWindowText(m_sOptimizerHints);
	m_ctrlFilter.GetWindowText(m_sFilter);
	
	m_dwStyle = NULL;
	if (m_ctrlFixedColumns.GetCheck())
		m_dwStyle |= TABLE_FIXEDCOLUMNS;
	if (m_ctrlShowVertGridLines.GetCheck())
		m_dwStyle |= TABLE_VERTGRIDLINES;

	m_bEnableIteratorBar = m_ctrlEnableIteratorBar.GetCheck();
	
	CDialog::OnOK();
}


void COpenDlg::OnCancel() 
{
	CDialog::OnCancel();
}


BOOL COpenDlg::CheckFormat(CString& sFormat)
{
	sFormat.TrimRight();
	
	// acceptable formats
	CString sFormatTypes(_T("cCdiouxXeEfgGsS"));

	if ((sFormat.IsEmpty()) ||
		(sFormat.Left(1) != _T("%")) ||
		(sFormatTypes.Find(sFormat.Right(1)) == -1))	// format not found
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}
