#include "stdafx.h"
#include "SQLViewApp.h"

#include "SQLTableDoc.h"
#include "SQLTableView.h"

#include "OpenDlg.h"
#include "SeekItemDlg.h"
#include "SetFilterDlg.h"
#include "SetSortDlg.h"
#include "EnableIteratorBarDlg.h"
#include "ErrorBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CSQLTableDoc, CTableDoc)


BEGIN_MESSAGE_MAP(CSQLTableDoc, CTableDoc)
	//{{AFX_MSG_MAP(CSQLTableDoc)
	ON_COMMAND(ID_OPEN, OnOpen)
	ON_COMMAND(ID_CLOSE, OnClose)
	ON_COMMAND(ID_SEEKITEM, OnCallSeekItem)
	ON_COMMAND(ID_SETFILTER, OnCallSetFilter)
	ON_COMMAND(ID_SETSORT, OnCallSetSort)
	ON_COMMAND(ID_ENABLEITERATORBAR, OnCallEnableIteratorBar)
	ON_UPDATE_COMMAND_UI(ID_OPEN, OnUpdateOpen)
	ON_UPDATE_COMMAND_UI(ID_CLOSE, OnUpdate)
	ON_UPDATE_COMMAND_UI(ID_SEEKITEM, OnUpdate)
	ON_UPDATE_COMMAND_UI(ID_SETFILTER, OnUpdate)
	ON_UPDATE_COMMAND_UI(ID_SETSORT, OnUpdate)
	ON_UPDATE_COMMAND_UI(ID_ENABLEITERATORBAR, OnUpdate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


CSQLTableDoc::CSQLTableDoc() :
	m_pSQLTableView(NULL)
{
	m_pRecordSet.CreateInstance(__uuidof(Recordset));
	m_pRecordSet->PutRefActiveConnection(static_cast<CSQLViewApp*>(AfxGetApp())->m_pConnection);
}


CSQLTableDoc::~CSQLTableDoc()
{
}


BOOL CSQLTableDoc::OnNewDocument()
{
	if (!CTableDoc::OnNewDocument())
		return FALSE;
	
	POSITION Position = GetFirstViewPosition();
	m_pSQLTableView = static_cast<CSQLTableView*>(GetNextView(Position));
	static_cast<CSQLViewApp*>(AfxGetApp())->m_pSQLTableView = m_pSQLTableView;

	m_sEmptyTitle = GetTitle();

	return TRUE;
}


#ifdef _DEBUG
void CSQLTableDoc::AssertValid() const
{
	CTableDoc::AssertValid();
}


void CSQLTableDoc::Dump(CDumpContext& dc) const
{
	CTableDoc::Dump(dc);
}
#endif //_DEBUG


void CSQLTableDoc::OpenTable() 
{
	OnOpen();
}


void CSQLTableDoc::OnOpen()
{
	COpenDlg dlgOpen(NULL, this);
	if (dlgOpen.DoModal() == IDOK)
	{
		int iColumnsNum = dlgOpen.m_arrColumns.GetSize();
		COpenDlg::CColumnData ColumnData;
		
		for (int i = 0; i < iColumnsNum; i++)
		{
			ColumnData = dlgOpen.m_arrColumns.GetAt(i);
			
			m_pSQLTableView->InsertColumn(ColumnData.sFieldName,
				ColumnData.sFieldFormat,
				ColumnData.sHeading,
				ColumnData.iFormat,
				ColumnData.iWidth);
		}

		m_pSQLTableView->Open(dlgOpen.m_sTableName,
			dlgOpen.m_sDataFieldName,
			dlgOpen.m_sSort,
			dlgOpen.m_sSortFormat,
			dlgOpen.m_sOptimizerHints,
			dlgOpen.m_sFilter,
			dlgOpen.m_dwStyle,
			dlgOpen.m_bEnableIteratorBar);

		SetTitle(dlgOpen.m_sTableName);
	}
}


void CSQLTableDoc::OnClose() 
{
	if (m_pSQLTableView->IsOpen())
	{
		m_pSQLTableView->Close();
		SetTitle(m_sEmptyTitle);
	}
}


void CSQLTableDoc::OnCallSeekItem() 
{
	CSeekItemDlg dlgSeekItem;

	if (dlgSeekItem.DoModal() == IDOK)
	{
		CTableView::CItem Item;

		if (m_pSQLTableView->SeekItem(dlgSeekItem.vaData, &Item))
			m_pSQLTableView->GetPage(Item.vaData, Item.dwData);
		else
			CErrorBox("", _T("Item not found"), "");
	}
}


void CSQLTableDoc::OnCallSetFilter()
{
	CSetFilterDlg dlgSetFilter;

	if (dlgSetFilter.DoModal() == IDOK)
	{
		m_pSQLTableView->SetFilter(dlgSetFilter.m_sFilter);
	}
}


void CSQLTableDoc::OnCallSetSort()
{
	CSetSortDlg dlgSetSort;

	if (dlgSetSort.DoModal() == IDOK)
	{
		m_pSQLTableView->SetSort(dlgSetSort.m_sSort,
			dlgSetSort.m_sSortFormat, dlgSetSort.m_sOptimizerHints);
	}
}


void CSQLTableDoc::OnCallEnableIteratorBar() 
{
	CEnableIteratorBarDlg dlgEnableIteratorBar;

	if (dlgEnableIteratorBar.DoModal() == IDOK)
	{
		m_pSQLTableView->EnableIteratorBar(dlgEnableIteratorBar.m_bEnable);
	}
}


void CSQLTableDoc::OnUpdateOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_pSQLTableView->IsOpen());
}


void CSQLTableDoc::OnUpdate(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSQLTableView->IsOpen());
}


void CSQLTableDoc::ProcessError(_com_error& e)
{
	CErrorBox(e.Description(), e.ErrorMessage(), e.Source());
	ExitProcess(0);
}


BOOL CSQLTableDoc::Open(LPCTSTR lpszQuery, 
	DWORD dwRowCount, 
	enum CursorTypeEnum enCursorType, 
	enum LockTypeEnum enLockType, 
	long lCacheSize)
{
	BOOL bResult = TRUE;
	
	CString sRowCount;
	sRowCount.Format(_T("SET ROWCOUNT %d\n\r"), dwRowCount);

	CString sQuery = sRowCount + CString(lpszQuery);

    _variant_t vNull;
	vNull.vt = VT_ERROR;
    vNull.scode = DISP_E_PARAMNOTFOUND;

	try
	{
		m_pRecordSet->Open(_bstr_t(sQuery), vNull, enCursorType, enLockType, adCmdText);
		m_pRecordSet->CacheSize = lCacheSize;

		if (GetBOF() && GetEOF())
			bResult = FALSE;	// RecordSet is empty
	}
	catch(_com_error& e)
	{
		ProcessError(e);
	}

	return bResult;
}


void CSQLTableDoc::Close()
{
	try
	{
		m_pRecordSet->Close();
	}
	catch(_com_error& e)
	{
		ProcessError(e);
	}
}


_variant_t CSQLTableDoc::GetField(LPCTSTR lpszField)
{
	_variant_t vaResult;
	
	try
	{
		if (!GetBOF() && !GetEOF())
			vaResult = m_pRecordSet->GetCollect(_bstr_t(lpszField));
	}
	catch(_com_error& e)
	{
		ProcessError(e);
	}

	// instead of VT_NULL return VT_EMPTY
	if (vaResult.vt == VT_NULL)
		vaResult.Clear();

	return vaResult;
}


BOOL CSQLTableDoc::GetBOF()
{
	BOOL bResult;
	
	try
	{
		bResult = m_pRecordSet->GetBOF();
	}
	catch(_com_error& e)
	{
		ProcessError(e);
	}

	return bResult;
}


BOOL CSQLTableDoc::GetEOF()
{
	BOOL bResult;
	
	try
	{
		bResult = m_pRecordSet->GetadoEOF();
	}
	catch(_com_error& e)
	{
		ProcessError(e);
	}

	return bResult;
}


BOOL CSQLTableDoc::MoveNext()
{
	BOOL bResult = TRUE;

	try
	{
		m_pRecordSet->MoveNext();
		
		if (GetEOF())
			bResult = FALSE;
	}
	catch(_com_error& e)
	{
		ProcessError(e);
	}

	return bResult;
}