#include "stdafx.h"
#include "resource.h"
#include "TableDoc.h"
#include "TableView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CTableView, CListView)


BEGIN_MESSAGE_MAP(CTableView, CListView)
	//{{AFX_MSG_MAP(CTableView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_PREV_ITEM, GetPrevItem)
	ON_COMMAND(ID_PREV_PAGE, GetPrevPage)
	ON_COMMAND(ID_HEAD, GetHead)
	ON_COMMAND(ID_TAIL, GetTail)
	ON_COMMAND(ID_NEXT_PAGE, GetNextPage)
	ON_COMMAND(ID_NEXT_ITEM, GetNextItem)
END_MESSAGE_MAP()


CTableView::CTableView() :
	m_Ctrl(GetListCtrl()),
	m_bInit(TRUE),
	m_bOpened(FALSE),
	m_dwStyle(NULL)
{
}


CTableView::~CTableView()
{
	m_lstFields.RemoveAll();
	m_lstItemCache.RemoveAll();
	m_mapItemData.RemoveAll();
}


#ifdef _DEBUG
void CTableView::AssertValid() const
{
	CListView::AssertValid();
}


void CTableView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}


CTableDoc* CTableView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTableDoc)));
	return static_cast<CTableDoc*>(m_pDocument);
}
#endif //_DEBUG


// --------- Public operations ---------
BOOL CTableView::InsertColumn(LPCTSTR lpszFieldName,
	LPCTSTR lpszFieldFormat,
	LPCTSTR lpszColumnHeading,
	int iColumnFormat,
	int iColumnWidth)
{
	// Column to insert (next column number)
	int iColumn = m_lstFields.GetCount();

	int iResult = m_Ctrl.InsertColumn(iColumn, lpszColumnHeading,
		iColumnFormat, iColumnWidth, /* iSubItem */ iColumn);

	if (iResult != -1)
	{
		CFieldData FieldData;
		FillString(FieldData.sFieldName, lpszFieldName);
		FillString(FieldData.sFieldFormat, lpszFieldFormat);

		m_lstFields.AddTail(FieldData);

		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


BOOL CTableView::Open(LPCTSTR lpszTableName,
	LPCTSTR lpszDataFieldName,
	LPCTSTR lpszSort,
	LPCTSTR lpszSortFormat,
	LPCTSTR lpszOptimizerHints,
	LPCTSTR lpszFilter,
	DWORD dwStyle,
	BOOL bEnableIteratorBar)
{
	if (m_lstFields.GetCount() == 0)
		return FALSE;	// No columns were inserted

	if (IsOpen())
		return FALSE;	// TableView was already opened

	if (!FillString(m_sTableName, lpszTableName))
		return FALSE;

	if (!FillString(m_sDataFieldName, lpszDataFieldName))
		return FALSE;

	if (!FillString(m_sSort, lpszSort))
		return FALSE;

	if (!FillString(m_sSortFormat, lpszSortFormat))
		return FALSE;

	FillString(m_sOptimizerHints, lpszOptimizerHints);

	FillString(m_sFilter, lpszFilter);

	m_dwStyle = dwStyle;

	// To prevent columns' resizing
	if (m_dwStyle & TABLE_FIXEDCOLUMNS)
		m_Ctrl.GetHeaderCtrl()->EnableWindow(FALSE);
	else
		m_Ctrl.GetHeaderCtrl()->EnableWindow(TRUE);

	if (bEnableIteratorBar)
		EnableIteratorBar(TRUE);
	else
		EnableIteratorBar(FALSE);

	m_bOpened = TRUE;

	GetHead();

	return TRUE;
}


void CTableView::Close()
{
	m_bOpened = FALSE;

	EnableIteratorBar(FALSE);

	m_Ctrl.DeleteAllItems();
	
	int iColumnsNum = m_lstFields.GetCount();
	for (int j = iColumnsNum - 1; j >= 0; j--)
		m_Ctrl.DeleteColumn(j);
	
	m_lstFields.RemoveAll();
	m_lstItemCache.RemoveAll();
	m_mapItemData.RemoveAll();
}


BOOL CTableView::IsOpen() const
{
	return m_bOpened;
}


BOOL CTableView::SetSort(LPCTSTR lpszSort,
	LPCTSTR lpszSortFormat,
	LPCTSTR lpszOptimizerHints)
{
	if (!IsOpen())
		return FALSE;

	CString sSort, sSortFormat;
	
	if (!FillString(sSort, lpszSort))
		return FALSE;

	if (!FillString(sSortFormat, lpszSortFormat))
		return FALSE;

	// sSort & sSortFormat are not empty
	m_sSort = sSort;
	m_sSortFormat = sSortFormat;

	FillString(m_sOptimizerHints, lpszOptimizerHints);

	DWORD dwData = m_Ctrl.GetItemData(GetCurrentItem());

	CString sQuery = BuildQuery(_T("ASC"), "", _variant_t(), _T("="), dwData);
	if (FillCache(TABLE_FORWARD, sQuery, 1))
	{
		POSITION Position = m_lstItemCache.GetHeadPosition();
		CItem Item = m_lstItemCache.GetNext(Position);

		GetPage(Item.vaData, Item.dwData);
	}

	return TRUE;
}



BOOL CTableView::SetFilter(LPCTSTR lpszFilter)
{
	if (!IsOpen())
		return FALSE;
	
	FillString(m_sFilter, lpszFilter);

	GetCurrentPage();

	return TRUE;
}


void CTableView::GetSort(CString& rsSort,
	CString& rsSortFormat,
	CString& rsOptimizerHints) const
{
	rsSort = m_sSort;
	rsSortFormat = m_sSortFormat;
	rsOptimizerHints = m_sOptimizerHints;
}


void CTableView::GetFilter(CString& rsFilter) const
{
	rsFilter = m_sFilter;
}


BOOL CTableView::EnableIteratorBar(BOOL bEnable)
{
	if (bEnable)
	{
		if (m_IteratorBar.GetSafeHwnd())
			return TRUE;	// already exists

		m_IteratorBar.CreateEx(GetParentFrame(), TBSTYLE_FLAT | TBSTYLE_TOOLTIPS, 
			WS_CHILD | WS_VISIBLE | CBRS_RIGHT, CRect(0, 0, 0, 0), AFX_IDW_TOOLBAR);
		ResizeIteratorBar();
		m_IteratorBar.LoadToolBar(IDB_ITERATOR);

		CBitmap bmpIterator;
		bmpIterator.LoadBitmap(IDB_ITERATOR);
		m_IteratorBarImageList.Create(24, 24, ILC_COLOR | ILC_MASK, 1, 1);
		m_IteratorBarImageList.Add(&bmpIterator, 0xFFFFFF);	// White mask
		m_IteratorBar.GetToolBarCtrl().SetImageList(&m_IteratorBarImageList);

		CBitmap bmpIteratorHot;
		bmpIteratorHot.LoadBitmap(IDB_ITERATOR_HOT);		
		m_IteratorBarHotImageList.Create(24, 24, ILC_COLOR | ILC_MASK, 1, 1);
		m_IteratorBarHotImageList.Add(&bmpIteratorHot, 0xFFFFFF);
		m_IteratorBar.GetToolBarCtrl().SetHotImageList(&m_IteratorBarHotImageList);

		GetParentFrame()->RecalcLayout();

		return TRUE;
	}
	else
	{
		if (!m_IteratorBar.GetSafeHwnd())
			return FALSE;	// already does not exist

		m_IteratorBarImageList.DeleteImageList();
		m_IteratorBarHotImageList.DeleteImageList();
		
		m_IteratorBar.DestroyWindow();
			
		GetParentFrame()->RecalcLayout();
		
		return FALSE;
	}
}


BOOL CTableView::SeekItem(_variant_t vaData, CItem* pItem)
{
	CWaitCursor WaitCursor;
	
	CString sType = m_sSortFormat.Right(1);

	if ((sType == _T("s")) || (sType == _T("S")))
	{
		CString sData = ExtractVariant(vaData, m_sSortFormat) + 
			_T("%");
		vaData = _variant_t(sData);
		
		// SELECT * FROM Table WHERE LTRIM(OrderData) 
		// LIKE 'extract(vaData)%'
		CString sQuery = BuildQuery(_T("ASC"), _T("LIKE"), vaData, "", NULL, _T("LTRIM"));
		if (FillCache(TABLE_FORWARD, sQuery, 1))
		{
			*pItem = m_lstItemCache.GetTail();
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
	else
	{
		// SELECT * FROM Table WHERE OrderData = extract(vaData)
		CString sQuery = BuildQuery(_T("ASC"), _T("="), vaData);
		if (FillCache(TABLE_FORWARD, sQuery, 1))
		{
			*pItem = m_lstItemCache.GetTail();
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
}


void CTableView::Update()
{
	// must be called from CWinApp::OnIdle()
	if (m_bInit)
	{
		// first time initialization
		m_bInit = FALSE;
		m_iItemsPerPage = m_Ctrl.GetCountPerPage();
		return;
	}
	
	int iItemsPerPage = m_Ctrl.GetCountPerPage();

	if (iItemsPerPage == m_iItemsPerPage)
		return;

	if (iItemsPerPage < m_iItemsPerPage)
	{
		// Window height decreased,
		// so last xxx items will be deleted.

		// There must be at least 1 item
		// in the control
		if (m_Ctrl.GetItemCount() < 2)
			return;

		int iCurrentItem = GetCurrentItem();

		// item 0 won't be deleted in any case
		for (int i = m_Ctrl.GetItemCount() - 1; i > 0; i--)
		{
			if (!IsItemVisible(i))
				m_Ctrl.DeleteItem(i);
		}

		int iItemCount = m_Ctrl.GetItemCount();
		if (iItemCount > 0)
		{
			if (iCurrentItem > (iItemCount - 1))
				SetCurrentItem(iItemCount - 1);
		}
	}
	else
	{
		// Window height increased
		// iItemsPerPage > m_iItemsPerPage
		GetCurrentPage();
	}

	m_iItemsPerPage = iItemsPerPage;
}


int CTableView::GetCurrentItem()
{
	return m_Ctrl.GetNextItem(-1, LVIS_SELECTED | LVIS_FOCUSED);
}


void CTableView::SetCurrentItem(int iCurrentItem)
{
	m_Ctrl.SetItemState(iCurrentItem, LVIS_SELECTED | LVIS_FOCUSED, 
		LVIS_SELECTED | LVIS_FOCUSED);
}


BOOL CTableView::IsItemVisible(int iItem)
{
	RECT rectClient;
	m_Ctrl.GetClientRect(&rectClient);

	RECT rectItem;
	m_Ctrl.GetItemRect(iItem, &rectItem, LVIR_BOUNDS);

	if (rectItem.bottom <= rectClient.bottom)
		return TRUE;
	else
		return FALSE;
}


// --------- Message handling ---------
BOOL CTableView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL;

	return CListView::PreCreateWindow(cs);
}


int CTableView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CListView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// This modification must be done exactly here
	// Otherwise header will not be shown (?)
	m_Ctrl.ModifyStyle(NULL, LVS_NOSCROLL);
	
	m_Ctrl.SetExtendedStyle(LVS_EX_FULLROWSELECT);
	m_Ctrl.SetBkColor(::GetSysColor(COLOR_WINDOW));

	return 0;
}


void CTableView::OnSize(UINT nType, int cx, int cy)
{
	CListView::OnSize(nType, cx, cy);
	
	// This modification must be done exactly here
	// Otherwise scroll bars will be visible (?)
	if (!IsOpen())
		ModifyStyle(WS_VSCROLL | WS_HSCROLL, NULL);

	ResizeHeader();
	ResizeIteratorBar();
}


void CTableView::OnPaint() 
{
	// Default painting
	const MSG* Msg = GetCurrentMessage();
	DefWindowProc(Msg->message, Msg->wParam, Msg->lParam);

	if (!IsOpen())
		return;

	if (m_dwStyle & TABLE_VERTGRIDLINES)
	{
		RECT rectClient;
		GetClientRect(&rectClient);

		RECT rectHeader;
		m_Ctrl.GetHeaderCtrl()->GetClientRect(&rectHeader);

		CClientDC dc(this);
		int iColumnsNum = m_lstFields.GetCount();

		for (int j = 0; j < iColumnsNum; j++)
		{
			int iX = GetColumnRightX(j) + 1;

			dc.MoveTo(iX, rectHeader.bottom);
			dc.LineTo(iX, rectClient.bottom);
		}
	}
}


void CTableView::OnDraw(CDC* pDC)
{
}


void CTableView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nRepCnt == 1)
	{
		switch (nChar)
		{
		case VK_UP:
		case VK_LEFT:
			GetPrevItem();
			return;

		case VK_PRIOR:
			GetPrevPage();
			return;

		case VK_HOME:
			GetHead();
			return;

		case VK_END:
			GetTail();
			return;

		case VK_NEXT:
			GetNextPage();
			return;

		case VK_DOWN:
		case VK_RIGHT:
			GetNextItem();
			return;
		}
	}

	CListView::OnKeyDown(nChar, nRepCnt, nFlags);
}


// --------- Private operations ---------
BOOL CTableView::FillString(CString& sDestination, LPCTSTR lpszSource)
{
	sDestination = lpszSource;
	sDestination.TrimLeft();
	sDestination.TrimRight();
	
	if (!sDestination.IsEmpty())
		return TRUE;
	else
		return FALSE;
}


void CTableView::ResizeHeader()
{
	CHeaderCtrl* pHeaderCtrl = m_Ctrl.GetHeaderCtrl();
	
	if (!(pHeaderCtrl->GetSafeHwnd()))
		return;

	RECT rectClient;
	GetClientRect(&rectClient);

	RECT rectHeader;

	pHeaderCtrl->GetClientRect(&rectHeader);

	rectHeader.left = rectClient.left;
	rectHeader.right = rectClient.right;

	pHeaderCtrl->MoveWindow(&rectHeader);
}


void CTableView::ResizeIteratorBar()
{
	if (!(m_IteratorBar.GetSafeHwnd()))
		return;
	
	CRect Rect;
	GetClientRect(&Rect);

	int iButtonHeight = 30;
	if (Rect.Height() / 6 > iButtonHeight)
		iButtonHeight = Rect.Height() / 6;

	m_IteratorBar.SetSizes(CSize(31, iButtonHeight), CSize(24, 24));
}


int CTableView::GetColumnLeftX(int iColumn)
{
	if (iColumn < 0)
		return 0;
	
	// Returns left border of the column
	int iAccumulativeColumnsWidth = 0;
			
	for (int j = 0; j < iColumn; j++)
		iAccumulativeColumnsWidth += m_Ctrl.GetColumnWidth(j);

	return iAccumulativeColumnsWidth;
}


int CTableView::GetColumnRightX(int iColumn)
{
	if (iColumn < 0)
		return 0;
	
	// Returns right border of the column
	int iAccumulativeColumnsWidth = 0;
			
	for (int j = 0; j <= iColumn; j++)
		iAccumulativeColumnsWidth += m_Ctrl.GetColumnWidth(j);

	return iAccumulativeColumnsWidth - 1;
}


int CTableView::GetItemCount()
{
	int iItemCount = m_Ctrl.GetItemCount();
	int iCountPerPage = m_Ctrl.GetCountPerPage();// - 1;

	if ((iItemCount == 0) || (iItemCount < iCountPerPage))
		iItemCount = iCountPerPage;

	return iItemCount;
}


// --------- Grid engine ---------
CString CTableView::ExtractVariant(_variant_t& va, LPCTSTR lpszFormat)
{
	// Most right symbol is a type format field
	CString sType = (CString(lpszFormat)).Right(1);

	CString sResult(_T("Invalid data or format specification"));

	if ((sType == _T("c")) || (sType == _T("C")))
	{
		// BYTE extractor will be used
		sResult.Format(lpszFormat, BYTE(va));
		return sResult;	
	}
	
	if ((sType == _T("d")) || (sType == _T("i")) ||
		(sType == _T("o")) || (sType == _T("u")) ||
		(sType == _T("x")) || (sType == _T("X")))
	{
		// long extractor will be used
		sResult.Format(lpszFormat, long(va));
		return sResult;
	}

	if ((sType == _T("e")) || (sType == _T("E")) ||
		(sType == _T("f")) || (sType == _T("g")) ||
		(sType == _T("G")))
	{
		// double extractor will be used
		sResult.Format(lpszFormat, double(va));
		return sResult;
	}

	if ((sType == _T("s")) || (sType == _T("S")))
	{
		// _bstr_t extractor will be used
		sResult.Format(lpszFormat, LPCTSTR(_bstr_t(va)));
		return sResult;
	}

	return sResult;
}


CString CTableView::BuildQuery(CString sSortOrder,
	CString svaDataRelation,
	_variant_t vaData,
	CString sdwDataRelation,
	DWORD dwData,
	CString sSortPrefix)
{
	// Important notice:
	// ">", "ASC" will be included in comments 
	// for simplification only,
	// they also can be "<" and "DESC"

	// --------------- Select ------------------
	// SELECT *,
	// (<m_sSort>) AS 'Sort by (<m_sSort>)'
	// FROM <m_sTableName> 
	CString sSelect = _T("SELECT *, (") +
		m_sSort +
		_T(") AS 'Sort by (") +
		m_sSort +
		_T(")' FROM ") +
		m_sTableName;
	
	if (!m_sOptimizerHints.IsEmpty())
	{ 
		sSelect += _T(" ") +
			m_sOptimizerHints;
	}
	// --------------- End Select ------------------

	// --------------- Where clause ------------------	
	CString sFilter;	// WHERE will be inserted at the end
	
	// F.e., prefix can be LTRIM()
	if (!sSortPrefix.IsEmpty())
	{
		sFilter += sSortPrefix +
			_T("(");
	}

	if (!svaDataRelation.IsEmpty())
	{
		// SELECT *,
		// (<m_sSort>) AS 'Sort by (<m_sSort>)'
		// FROM <m_sTableName> <m_sOptimizerHints>
		// WHERE <sSortPrefix>((m_sSort)
		sFilter += _T("(") +
			m_sSort +
			_T(")");
	}

	if (!sSortPrefix.IsEmpty())
	{
		sFilter += _T(")");
	}

	if (!svaDataRelation.IsEmpty())
	{
		// SELECT *,
		// (<m_sSort>) AS 'Sort by (<m_sSort>)'
		// FROM <m_sTableName> <m_sOptimizerHints>
		// WHERE <sSortPrefix>((m_sSort)) >
		sFilter += _T(" ") +
			svaDataRelation +
			_T(" ");

		CString sType = m_sSortFormat.Right(1);

		if ((sType == _T("s")) || (sType == _T("S")))
		{
			sFilter += _T("'");
		}

		// SELECT *,
		// (<m_sSort>) AS 'Sort by (<m_sSort>)'
		// FROM <m_sTableName> <m_sOptimizerHints>
		// WHERE <sSortPrefix>((m_sSort)) > extract(vaData)
		sFilter += ExtractVariant(vaData, m_sSortFormat);

		if ((sType == _T("s")) || (sType == _T("S")))
		{
			sFilter += _T("'");
		}
	}

	CString sData;
	sData.Format(_T("%d"), dwData);

	if ((!sdwDataRelation.IsEmpty()) && (!sData.IsEmpty()))
	{
		if (!sFilter.IsEmpty())
		{
			sFilter += _T(" AND ");
		}
		
		// SELECT *,
		// (<m_sSort>) AS 'Sort by (<m_sSort>)'
		// FROM <m_sTableName> <m_sOptimizerHints>
		// WHERE <sSortPrefix>((m_sSort)) > extract(vaData)
		// AND <m_sDataFieldName> > <sData>
		sFilter += m_sDataFieldName +
			_T(" ") +
			sdwDataRelation +
			_T(" ") +
			sData;
	}

	if (!m_sFilter.IsEmpty())
	{
		if (!sFilter.IsEmpty())
		{
			sFilter += _T(" AND ");
		}

		// SELECT *,
		// (<m_sSort>) AS 'Sort by (<m_sSort>)'
		// FROM <m_sTableName> <m_sOptimizerHints>
		// WHERE <sSortPrefix>((m_sSort)) > extract(vaData)
		// AND <m_sDataFieldName> > <sData>
		// AND <m_sFilter>
		sFilter += m_sFilter;
	}

	if (!sFilter.IsEmpty())
	{
		sFilter.Insert(0, _T(" WHERE "));
	}
	// --------------- End Where clause ------------------	
	
	// --------------- OrderBy clause ------------------	
	// SELECT *, 
	// (<m_sSort>) AS 'Sort by (<m_sSort>)'
	// FROM <m_sTableName> <m_sOptimizerHints>
	// WHERE <sSortPrefix>((m_sSort)) > extract(vaData)
	// AND <m_sDataFieldName> > <sData>
	// AND <m_sFilter>
	// ORDER BY (<m_sSort>) ASC, <m_sDataFieldName> ASC
	CString sSort = _T(" ORDER BY (") + 
		m_sSort +
		_T(") ") +
		sSortOrder +
		_T(", ") +
		m_sDataFieldName +
		_T(" ") +
		sSortOrder;
	// --------------- End OrderBy clause ------------------	

	CString sQuery = sSelect + sFilter + sSort;

	return sQuery;
}


BOOL CTableView::FillCache(DWORD fdwOrder,
	LPCTSTR lpszQuery,
	DWORD dwRowCount,
	BOOL bEraseCache)
{
	if (bEraseCache)
		m_lstItemCache.RemoveAll();

	BOOL bResult = FALSE;

	CString sSortName = _T("Sort by (") +
		m_sSort +
		_T(")");
	
	GetDocument()->Open(lpszQuery, dwRowCount);
	
	while (!GetDocument()->GetEOF())
	{
		CItem Item;

		POSITION Position = m_lstFields.GetHeadPosition();
		while (Position)
		{
			CFieldData FieldData = m_lstFields.GetNext(Position);
			
			_variant_t vaField = GetDocument()->GetField(FieldData.sFieldName);
			CString sField = ExtractVariant(vaField, FieldData.sFieldFormat);

			Item.m_lstText.AddTail(sField);
		}

		Item.dwData = long(GetDocument()->GetField(m_sDataFieldName));
		Item.vaData = GetDocument()->GetField(sSortName);

		switch (fdwOrder)
		{
		case TABLE_FORWARD:
			m_lstItemCache.AddTail(Item);
			break;
		case TABLE_BACKWARD:
			m_lstItemCache.AddHead(Item);
			break;
		}

		GetDocument()->MoveNext();

		bResult = TRUE;
	}
	
	GetDocument()->Close();

	if (m_lstItemCache.IsEmpty())
		bResult = FALSE;

	// bResult == FALSE, if no item was inserted into cache,
	// OR whyever cache is empty
	// So, bResult == TRUE, if some items were inserted into cache
	// AND cache is not empty
	return bResult;
}


BOOL CTableView::FillListCtrl()
{
	BOOL bResult = FALSE;

	m_Ctrl.DeleteAllItems();
	m_mapItemData.RemoveAll();
	
	CItem Item;
	POSITION Position = m_lstItemCache.GetHeadPosition();
	
	for (int i = 0; Position; i++)
	{
		Item = m_lstItemCache.GetNext(Position);

		// Insert new item
		m_Ctrl.InsertItem(i, "");
		
		{
			POSITION Position = Item.m_lstText.GetHeadPosition();
			for (int j = 0; Position; j++)
				m_Ctrl.SetItemText(i, j, Item.m_lstText.GetNext(Position));
		}

		m_Ctrl.SetItemData(i, Item.dwData);
		m_mapItemData.SetAt(Item.dwData, Item.vaData);

		bResult = TRUE;
	}

	// bResult == TRUE, if some items 
	// were inserted into the control
	return bResult;
}


BOOL CTableView::GetPage(_variant_t vaData, DWORD dwData)
{
	if (!IsOpen())
		return FALSE;
	
	// Returns FALSE if no records were inserted
	// into the control
	// TRUE if were inserted
	
	int iItemCount = m_Ctrl.GetItemCount();

	CWaitCursor WaitCursor;

	// This item (if exists) will be LAST on the page
	CItem Item;
	Item.dwData = dwData;
	Item.vaData = vaData;

	CItem ItemToSave = Item;

	m_lstItemCache.RemoveAll();

	CString sQuery = BuildQuery(_T("DESC"), _T("="), Item.vaData, _T("="), Item.dwData);
	if (!FillCache(TABLE_BACKWARD, sQuery, 1, FALSE))
		return FALSE;

	int i = 1; // Last item is already in cache
	for (; i < iItemCount; i++)
	{
		// First queries are made on the basis of supposition
		// that the previous lines have the same vaData as the inserted one
		// If it is so, the result of this queries will be right
		sQuery = BuildQuery(_T("DESC"), _T("<="), Item.vaData, _T("<"), Item.dwData);
		if (!FillCache(TABLE_BACKWARD, sQuery, 1, FALSE))
			break;

		Item = m_lstItemCache.GetHead();
		if (Item.vaData != ItemToSave.vaData)
		{
			// The result of last query is wrong!
			// So the last inserted item must be deleted from cache
			// "Previous" item will be retrieved by the second query
			m_lstItemCache.RemoveHead();
			break;
		}
	}

	if (i == iItemCount)
	{
		// This code must be executed if all the items 
		// on the page have the same text
		if (FillListCtrl())
			SetCurrentItem(iItemCount - 1);
		return TRUE;
	}

	sQuery = BuildQuery(_T("DESC"), _T("<"), ItemToSave.vaData);
	if (FillCache(TABLE_BACKWARD, sQuery, iItemCount - i, FALSE))
	{
		if (m_lstItemCache.GetCount() == iItemCount)
		{
			// This code must be executed if there are
			// <iItemCount - 1> or more items before the last one
			// on the page
			if (FillListCtrl())
				SetCurrentItem(iItemCount - 1);
			return TRUE;
		}
	}
		
	// This code must be executed if there are
	// less then <iItemCount - 1> items before the last one
	// on the page (head of table reached)
	GetHead();

	// Set current item (with ItemData == dwData 
	// that were passed to this function)
	LVFINDINFO LVFindInfo;
	LVFindInfo.flags = LVFI_PARAM;
	LVFindInfo.lParam = dwData;
	int iItem = m_Ctrl.FindItem(&LVFindInfo);
	if (iItem != -1)
		SetCurrentItem(iItem);

	return TRUE;
}


void CTableView::GetPrevItem()
{
	if (!IsOpen())
		return;
	
	if (m_Ctrl.GetItemCount() == 0)
		return;
	
	int iCurrentItem = GetCurrentItem();
	if (iCurrentItem != 0)
	{
		SetCurrentItem(--iCurrentItem);
		return;
	}

	CWaitCursor WaitCursor;

	CItem Item;
	Item.dwData = m_Ctrl.GetItemData(0);
	m_mapItemData.Lookup(Item.dwData, Item.vaData);
	CItem ItemToSave = Item;

	// First query is made on the basis of supposition
	// that the previous line has the same vaData as the current one
	// If it is so, the result of this query will be right
	CString sQuery = BuildQuery(_T("DESC"), _T("<="), Item.vaData, _T("<"), Item.dwData);
	if (FillCache(TABLE_BACKWARD, sQuery, 1))
	{
		Item = m_lstItemCache.GetTail();
		if (Item.vaData == ItemToSave.vaData)
		{
			ScrollBackward(Item);
			return;
		}
	}
		
	// This code must be executed in next 2 cases:
	// 1) Previous line has not the same vaData
	// If it is so, the result of previous query is wrong,
	// and in this case the previous line must be retrieved
	// by the next query
	// 2) The result of the first query is empty, so
	// the previous line must be retrieved
	// by the next query
	sQuery = BuildQuery(_T("DESC"), _T("<"), ItemToSave.vaData);
	if (FillCache(TABLE_BACKWARD, sQuery, 1))
		ScrollBackward(m_lstItemCache.GetTail());
}


void CTableView::ScrollBackward(CItem& ItemToAdd)
{
	int iItemCount = m_Ctrl.GetItemCount();

	DWORD dwData = m_Ctrl.GetItemData(iItemCount - 1);
	m_mapItemData.RemoveKey(dwData);

	for (int i = iItemCount - 1; i > 0; i--)
	{
		CItem Item;
		
		int iColumnsNum = m_lstFields.GetCount();
		for (int j = 0; j < iColumnsNum; j++)
			Item.m_lstText.AddTail(m_Ctrl.GetItemText(i - 1, j));
		Item.dwData = m_Ctrl.GetItemData(i - 1);

		POSITION Position = Item.m_lstText.GetHeadPosition();
		for (j = 0; Position; j++)
			m_Ctrl.SetItemText(i, j, Item.m_lstText.GetNext(Position));
		m_Ctrl.SetItemData(i, Item.dwData);
	}

	// Insert new item
	POSITION Position = ItemToAdd.m_lstText.GetHeadPosition();
	for (int j = 0; Position; j++)
		m_Ctrl.SetItemText(0, j, ItemToAdd.m_lstText.GetNext(Position));
	m_Ctrl.SetItemData(0, ItemToAdd.dwData);

	m_mapItemData.SetAt(ItemToAdd.dwData, ItemToAdd.vaData);
}


void CTableView::GetPrevPage()
{
	if (!IsOpen())
		return;
	
	if (m_Ctrl.GetItemCount() == 0)
		return;
	
	int iCurrentItem = GetCurrentItem();

	if (iCurrentItem != 0)
	{
		SetCurrentItem(0);
		return;
	}
	
	CWaitCursor WaitCursor;

	CItem Item;
	Item.dwData = m_Ctrl.GetItemData(0);
	m_mapItemData.Lookup(Item.dwData, Item.vaData);
	CItem ItemToSave = Item;

	m_lstItemCache.RemoveAll();

	int i = 0;
	int iItemCount = m_Ctrl.GetItemCount();
	for (; i < iItemCount; i++)
	{
		// First queries are made on the basis of supposition
		// that the previous lines have the same vaData as the current one
		// If it is so, the result of this queries will be right
		CString sQuery = BuildQuery(_T("DESC"), _T("<="), Item.vaData, _T("<"), Item.dwData);
		if (!FillCache(TABLE_BACKWARD, sQuery, 1, FALSE))
			break;

		Item = m_lstItemCache.GetHead();
		if (Item.vaData != ItemToSave.vaData)
		{
			// The result of last query is wrong!
			// So the last inserted item must be deleted from cache
			// "Previous" item will be retrieved by the second query
			m_lstItemCache.RemoveHead();
			break;
		}
	}

	if (i == iItemCount)
	{
		// This code must be executed if all the items 
		// on the previous page have the same vaData
		if (FillListCtrl())
			SetCurrentItem(0);
		return;
	}

	CString sQuery = BuildQuery(_T("DESC"), _T("<"), ItemToSave.vaData);
	if (FillCache(TABLE_BACKWARD, sQuery, iItemCount - i, FALSE))
	{
		if (m_lstItemCache.GetCount() == iItemCount)
		{
			// This code must be executed if there are
			// <iItemCount> or more items before the first one
			// on the current page
			if (FillListCtrl())
				SetCurrentItem(0);
			return;
		}
	}
		
	// This code must be executed if there are
	// less then <iItemCount> items before the first one
	// on the current page (head of table reached)
	GetHead();
}


void CTableView::GetHead()
{
	if (!IsOpen())
		return;
	
	CWaitCursor WaitCursor;

	CString sQuery = BuildQuery(_T("ASC"), _T(">="), _variant_t(), _T(">"), NULL);
	if (FillCache(TABLE_FORWARD, sQuery, GetItemCount()))
	{
		if (FillListCtrl())
			SetCurrentItem(0);
	}
}


void CTableView::GetTail()
{
	if (!IsOpen())
		return;
	
	CWaitCursor WaitCursor;

	CString sQuery = BuildQuery(_T("DESC"), _T(">="), _variant_t(), _T(">"), NULL);
	if (FillCache(TABLE_BACKWARD, sQuery, GetItemCount()))
	{
		if (FillListCtrl())
			SetCurrentItem(m_Ctrl.GetItemCount() - 1);
	}
}


void CTableView::GetNextPage()
{
	if (!IsOpen())
		return;
	
	int iItemCount = m_Ctrl.GetItemCount();

	if (iItemCount == 0)
		return;

	if (GetCurrentItem() != (iItemCount - 1))
	{
		SetCurrentItem(iItemCount - 1);
		return;
	}

	CWaitCursor WaitCursor;

	CItem Item;
	Item.dwData = m_Ctrl.GetItemData(iItemCount - 1);
	m_mapItemData.Lookup(Item.dwData, Item.vaData);
	CItem ItemToSave = Item;

	m_lstItemCache.RemoveAll();

	int i = 0;
	for (; i < iItemCount; i++)
	{
		// First queries are made on the basis of supposition
		// that the next lines have the same vaData as the current one
		// If it is so, the result of this queries will be right
		CString sQuery = BuildQuery(_T("ASC"), _T(">="), Item.vaData, _T(">"), Item.dwData);
		if (!FillCache(TABLE_FORWARD, sQuery, 1, FALSE))
			break;

		Item = m_lstItemCache.GetTail();
		if (Item.vaData != ItemToSave.vaData)
		{
			// The result of last query is wrong!
			// So the last inserted item must be deleted from cache
			// "Next" item will be retrieved by the second query
			m_lstItemCache.RemoveTail();
			break;
		}
	}

	if (i == iItemCount)
	{
		// This code must be executed if all the items 
		// on the next page have the same vaData
		if (FillListCtrl())
			SetCurrentItem(iItemCount - 1);
		return;
	}

	CString sQuery = BuildQuery(_T("ASC"), _T(">"), ItemToSave.vaData);
	if (FillCache(TABLE_FORWARD, sQuery, iItemCount - i, FALSE))
	{
		if (m_lstItemCache.GetCount() == iItemCount)
		{
			// This code must be executed if there are
			// <iItemCount> or more items after the last one
			// on the current page
			if (FillListCtrl())
				SetCurrentItem(iItemCount - 1);
			return;
		}
	}
		
	// This code must be executed if there are
	// less then <iItemCount> items after the last one
	// on the current page (end of table reached)
	GetTail();
}


void CTableView::GetNextItem()
{
	if (!IsOpen())
		return;
	
	if (m_Ctrl.GetItemCount() == 0)
		return;
	
	int iCurrentItem = GetCurrentItem();
	if (iCurrentItem != (m_Ctrl.GetItemCount() - 1))
	{
		SetCurrentItem(++iCurrentItem);
		return;
	}

	CWaitCursor WaitCursor;

	int iItemCount = m_Ctrl.GetItemCount();

	CItem Item;
	Item.dwData = m_Ctrl.GetItemData(iItemCount - 1);
	m_mapItemData.Lookup(Item.dwData, Item.vaData);
	CItem ItemToSave = Item;
	
	// First query is made on the basis of supposition
	// that the next line has the same vaData as the current one
	// If it is so, the result of this query will be right
	CString sQuery = BuildQuery(_T("ASC"), _T(">="), Item.vaData, _T(">"), Item.dwData);
	if (FillCache(TABLE_FORWARD, sQuery, 1))
	{
		Item = m_lstItemCache.GetHead();
		if (Item.vaData == ItemToSave.vaData)
		{
			ScrollForward(Item);
			return;
		}
	}

	// This code must be executed in next 2 cases:
	// 1) Next line has not the same vaData
	// If it is so, the result of previous query is wrong,
	// and in this case the next line must be retrieved
	// by the next query
	// 2) The result of the first query is empty, so
	// the next line must be retrieved
	// by the next query
	sQuery = BuildQuery(_T("ASC"), _T(">"), ItemToSave.vaData);
	if (FillCache(TABLE_FORWARD, sQuery, 1))
		ScrollForward(m_lstItemCache.GetHead());
}


void CTableView::ScrollForward(CItem& ItemToAdd)
{
	int iItemCount = m_Ctrl.GetItemCount();

	DWORD dwData = m_Ctrl.GetItemData(0);
	m_mapItemData.RemoveKey(dwData);

	for (int i = 1; i < iItemCount; i++)
	{
		CItem Item;

		int iColumnsNum = m_lstFields.GetCount();
		for (int j = 0; j < iColumnsNum; j++)
			Item.m_lstText.AddTail(m_Ctrl.GetItemText(i, j));
		Item.dwData = m_Ctrl.GetItemData(i);

		POSITION Position = Item.m_lstText.GetHeadPosition();
		for (j = 0; Position; j++)
			m_Ctrl.SetItemText(i - 1, j, Item.m_lstText.GetNext(Position));
		m_Ctrl.SetItemData(i - 1, Item.dwData);
	}

	// Insert new item
	POSITION Position = ItemToAdd.m_lstText.GetHeadPosition();
	for (int j = 0; Position; j++)
		m_Ctrl.SetItemText(iItemCount - 1, j, ItemToAdd.m_lstText.GetNext(Position));
	m_Ctrl.SetItemData(iItemCount - 1, ItemToAdd.dwData);

	m_mapItemData.SetAt(ItemToAdd.dwData, ItemToAdd.vaData);
}


void CTableView::GetCurrentPage()
{
	if (!IsOpen())
		return;
	
	// Re-reads current page
	// Called from Update() when window size increased,
	// Called from SetFilter() when this clause has changed
	
	if (m_Ctrl.GetItemCount() == 0)
		return;
	
	CWaitCursor WaitCursor;

	int iCurrentItem = GetCurrentItem();

	CItem Item;
	Item.dwData = m_Ctrl.GetItemData(0);
	m_mapItemData.Lookup(Item.dwData, Item.vaData);
	CItem ItemToSave = Item;

	m_lstItemCache.RemoveAll();

	CString sQuery = BuildQuery(_T("ASC"), _T("="), Item.vaData, _T("="), Item.dwData);
	FillCache(TABLE_FORWARD, sQuery, 1, FALSE);

	int i = 1;	// first item was already inserted by the prev. select
	int iItemCount = m_Ctrl.GetCountPerPage();
	for (; i < iItemCount; i++)
	{
		// First queries are made on the basis of supposition
		// that the next lines have the same vaData as the current one
		// If it is so, the result of this queries will be right
		sQuery = BuildQuery(_T("ASC"), _T(">="), Item.vaData, _T(">"), Item.dwData);
		if (!FillCache(TABLE_FORWARD, sQuery, 1, FALSE))
			break;

		Item = m_lstItemCache.GetTail();
		if (Item.vaData != ItemToSave.vaData)
		{
			// The result of last query is wrong!
			// So the last inserted item must be deleted from cache
			// "Next" item will be retrieved by the second query
			m_lstItemCache.RemoveTail();
			break;
		}
	}

	if (i == iItemCount)
	{
		// This code must be executed if all the items 
		// on the next page have the same vaData
		if (FillListCtrl())
			SetCurrentItem(iCurrentItem);
		return;
	}

	sQuery = BuildQuery(_T("ASC"), _T(">"), ItemToSave.vaData);
	if (FillCache(TABLE_FORWARD, sQuery, iItemCount - i, FALSE))
	{
		if (m_lstItemCache.GetCount() == iItemCount)
		{
			// This code must be executed if there are
			// <iItemCount> or more items after the last one
			// on the current page
			if (FillListCtrl())
				SetCurrentItem(iCurrentItem);
			return;
		}
	}
		
	// This code must be executed if there are
	// less then <iItemCount> items after the last one
	// on the current page (end of table reached)
	GetTail();
}


// --------- CTableView::CItem ---------
CTableView::CItem::CItem()
{
}


CTableView::CItem::CItem(const CItem& Item)
{
	dwData = Item.dwData;
	vaData = Item.vaData;
	
	m_lstText.RemoveAll();
	
	POSITION Position = Item.m_lstText.GetHeadPosition();
	while (Position)
		m_lstText.AddTail(Item.m_lstText.GetNext(Position));
}


CTableView::CItem::~CItem()
{
	m_lstText.RemoveAll();
}


CTableView::CItem& CTableView::CItem::operator=(const CItem& Item)
{
	dwData = Item.dwData;
	vaData = Item.vaData;

	m_lstText.RemoveAll();
	
	POSITION Position = Item.m_lstText.GetHeadPosition();
	while (Position)
		m_lstText.AddTail(Item.m_lstText.GetNext(Position));

	// Assignment operator returns left side.
	return *this;
}


CString CTableView::CItem::GetData()
{
	CString sData;
	sData.Format(_T("%d"), dwData);

	return sData;
}
