#if !defined(AFX_TABLEVIEW_H__46D3772C_7061_11D5_A5F1_00600816F009__INCLUDED_)
#define AFX_TABLEVIEW_H__46D3772C_7061_11D5_A5F1_00600816F009__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h>

#define TABLE_FORWARD	1
#define TABLE_BACKWARD	2

#define TABLE_FIXEDCOLUMNS			0x00000001
#define TABLE_VERTGRIDLINES			0x00000010


class CTableDoc;

class CTableView : public CListView
{
protected: // create from serialization only
	CTableView();
	DECLARE_DYNCREATE(CTableView)

// Attributes
public:
	CListCtrl& m_Ctrl;

	struct CItem
	{
		CItem();
		CItem(const CItem& Item);
		~CItem();
		CItem& operator=(const CItem& Item);

		CString GetData();
		
		DWORD dwData;
		_variant_t vaData;
		
		// For storing subitems' text
		CStringList m_lstText;
	};

// Operations
public:
	CTableDoc* GetDocument();

	BOOL InsertColumn(LPCTSTR lpszFieldName,
		LPCTSTR lpszFieldFormat,
		LPCTSTR lpszColumnHeading = "",
		int iColumnFormat = LVCFMT_LEFT, 
		int iColumnWidth = -1);

	BOOL Open(LPCTSTR lpszTableName,
		LPCTSTR lpszDataFieldName,
		LPCTSTR lpszSort,
		LPCTSTR lpszSortFormat,
		LPCTSTR lpszOptimizerHints = "",
		LPCTSTR lpszFilter = "",
		DWORD dwStyle = NULL,
		BOOL bEnableIteratorBar = TRUE);
	void Close();
	BOOL IsOpen() const;

	BOOL SetSort(LPCTSTR lpszSort,
		LPCTSTR lpszSortFormat,
		LPCTSTR lpszOptimizerHints);
	BOOL SetFilter(LPCTSTR lpszFilter);

	void GetSort(CString& rsSort,
		CString& rsSortFormat,
		CString& rsOptimizerHints) const;
	void GetFilter(CString& rsFilter) const;
	
	BOOL EnableIteratorBar(BOOL bEnable = TRUE);

	void GetCurrentPage();
	BOOL GetPage(_variant_t vaData, DWORD dwData);
	BOOL SeekItem(_variant_t vaData, CItem* pItem);

	void Update();

	int GetCurrentItem();
	void SetCurrentItem(int iCurrentItem);

	BOOL IsItemVisible(int iItem);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTableView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CTableView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

// Attributes
protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CTableView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnPaint();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	//}}AFX_MSG
	afx_msg void GetPrevItem();
	afx_msg void GetPrevPage();
	afx_msg void GetHead();
	afx_msg void GetTail();
	afx_msg void GetNextPage();
	afx_msg void GetNextItem();
	DECLARE_MESSAGE_MAP()

// Attributes
private:
	struct CFieldData
	{
		CString sFieldName;
		CString sFieldFormat;
	};

	// For storing fields' names & formats
	CList<CFieldData, CFieldData&> m_lstFields;

	// For storing items
	CList<CItem, CItem&> m_lstItemCache;
	
	// For storing values that are used 
	// when building a Sort clause of query
	CMap<DWORD, DWORD&, _variant_t, _variant_t&> m_mapItemData;

	DWORD m_dwStyle;

	BOOL m_bInit;
	BOOL m_bOpened;
	
	int m_iItemsPerPage;

	CString m_sTableName;
	CString m_sDataFieldName;
	CString m_sSort;
	CString m_sSortFormat;
	CString m_sOptimizerHints;
	CString m_sFilter;

	CToolBar m_IteratorBar;
	CImageList m_IteratorBarImageList, m_IteratorBarHotImageList;

// Operations
private:
	CString BuildQuery(CString sSortOrder,
	CString svaDataRelation,
	_variant_t vaData,
	CString sdwDataRelation = "",
	DWORD dwData = NULL,
	CString sSortPrefix = "");
	
	BOOL FillCache(DWORD fdwOrder,
		LPCTSTR lpszQuery,
		DWORD dwRowCount,
		BOOL bEraseCache = TRUE);

	BOOL FillListCtrl();

	CString ExtractVariant(_variant_t& va,
		LPCTSTR lpszFormat);
	
	int GetColumnLeftX(int iColumn);
	int GetColumnRightX(int iColumn);

	int GetItemCount();

	BOOL FillString(CString& sDestination, LPCTSTR lpszSource);

	void ResizeHeader();
	void ResizeIteratorBar();

	void ScrollForward(CItem& ItemToAdd);
	void ScrollBackward(CItem& ItemToAdd);
};

#ifndef _DEBUG  // debug version in TableView.cpp
inline CTableDoc* CTableView::GetDocument()
{
	return reinterpret_cast<CTableDoc*>(m_pDocument);
}
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TABLEVIEW_H__46D3772C_7061_11D5_A5F1_00600816F009__INCLUDED_)
